# frozen_string_literal: true

require 'spec_helper'

describe AuthStrategist::StrategyInterface do
  let(:strategy_class) { MissingAuthorized }

  context 'included' do
    it 'adds .define_components method to base' do
      expect(strategy_class).to respond_to(:define_components)
    end

    it 'adds .components method to base' do
      expect(strategy_class).to respond_to(:components)
    end

    it 'adds constructor (.new method) to base' do
      expect(strategy_class).to respond_to(:new)
    end

    it 'adds #authorize! method to base' do
      expect(strategy_class.new).to respond_to(:authorize!)
    end

    it 'adds #unauthorized method to base' do
      expect(strategy_class.new).to respond_to(:unauthorized)
    end

    describe '.new' do
      let(:options) { { user: 'john', password: 'rambo' } }

      before do
        strategy_class.define_components(*options.keys)
        options[:missing_component] = :missing_component
      end

      subject { strategy_class.new(options) }

      it 'assigns components values properly' do
        expect(subject.user).to eq('john')
        expect(subject.password).to eq('rambo')
        expect { subject.missing_component }.to raise_error(NoMethodError)
      end
    end

    describe '#authorize!' do
      subject { strategy_class.new }

      context 'when not implemented within base' do
        it 'raises error (AuthStrategist::Errors::AuthorizedNotImplemented)' do
          expect { subject.authorize! }
            .to raise_error(AuthStrategist::Errors::AuthorizedNotImplemented, 'MissingAuthorized#authorized? was not implemented.')
        end
      end

      context 'when implemented within base' do
        let(:strategy_class) { DummyStrategy }
        subject { strategy_class.new(dummy_component: :ok) }

        it 'executes like defined' do
          expect(subject.authorize!).to be_nil
        end
      end
    end

    describe '#unauthorized' do
      subject { strategy_class.new }

      it { expect(subject.unauthorized).to eq(AuthStrategist::Errors::Unauthorized) }
    end
  end
end
